var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
import BigNumber from 'bignumber.js';
import deepFreeze from 'deep-freeze';
import { oneLine } from 'common-tags';
import isInt from 'validator/lib/isInt';
import isFloat from 'validator/lib/isFloat';
import Currency from '../Currency';
import CurrencyMismatchError from '../errors/CurrencyMismatchError';
import WrongInputError from '../errors/WrongInputError';
import { convertBigNumberToStringInteger, getSmallestUnitAsBigNumber, getSmallestUnitDivisor, } from './fn/_internals';
import ROUNDING from '../constants/ROUNDING';
/**
 * @example
 * Basic usage
 * ```js
 * import { Money } from 'wealth'
 * let price = new Money('7856', 'USD'); // $78.56
 * ```
 */
var Money = /** @class */ (function () {
    /**
     * @example
     * Integer as value
     * ```js
     * let price = new Money(7856, 'USD'); // $78.56
     * ```
     *
     * @example
     * String integer as value
     * ```js
     * let price = new Money('7856', 'USD'); // $78.56
     * ```
     *
     * @example
     * String Float as value
     * ```js
     * let price = new Money('78.56', 'USD'); // $78.56
     * ```
     *
     * @example
     * Money instance as value
     * ```js
     * let price = new Money(new Money('78.56', 'USD'), 'USD'); // $78.56 - same as using clone()
     * ```
     *
     * @example
     * Currency instance as currency
     * ```js
     * let price = new Money('7856', new Currency('USD')); // $78.56
     * ```
     *
     * @param value - integer, integer string, float string, instance of `Money`
     * @param currency - currency code as string, currency settings or instance of `Currency`
     */
    function Money(value, currency) {
        this.currency = new Currency(currency);
        this.bigNumberConstructor = this.generateBigNumberConstructor();
        this.value = this.preProcessInputValue(value, this.bigNumberConstructor);
        deepFreeze(this);
    }
    /**
     * Checks if the current currency is the same as that of the parameter
     * @param value - value to check currency against the current value
     * @returns - true if the current value has the same currency as the parameter
     */
    Money.prototype.hasSameCurrency = function (value) {
        if (!(value instanceof Money)) {
            throw new WrongInputError('The input value must be a "Money" instance.');
        }
        return this.currency.is(value.currency);
    };
    /**
     * Create a new money instance, holding an identical value and currency to the current one
     * @returns - the cloned money instance
     */
    Money.prototype.clone = function () {
        var MoneyConstructor = this.constructor;
        return new MoneyConstructor(this.amount, this.currency);
    };
    Object.defineProperty(Money.prototype, "amountAsBigNumber", {
        /**
         * Get the current value as an instance of BigNumber
         * @returns - Internal BigNumber representation of the current value
         */
        get: function () {
            return this.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Money.prototype, "amountAsStringInteger", {
        /**
         * Get the current value as a string integer
         * @returns - String integer representation of the current value
         */
        get: function () {
            return convertBigNumberToStringInteger(this, this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Money.prototype, "amountAsStringFloat", {
        /**
         * Get the current value as a string float
         * @returns - String float representation of the current value
         */
        get: function () {
            return this.value.toFixed(this.currency.decimalDigits);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Money.prototype, "amount", {
        /**
         * Get the current value as a string float  (same as `amountAsStringFloat`)
         * @returns - String float representation of the current value
         */
        get: function () {
            return this.amountAsStringFloat;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get the current value as a string float (same as `amount`)
     * @returns - String float representation of the current value
     */
    Money.prototype.toString = function () {
        return this.amount;
    };
    Object.defineProperty(Money.prototype, "smallestUnit", {
        /**
         * Get the smallest unit of the current monetary value, i.e., 0.01 (aka penny) in a USD money
         * @returns - new Money instance holding the smallest unit of the current monetary value
         */
        get: function () {
            var MoneyConstructor = this.constructor;
            return new MoneyConstructor(getSmallestUnitAsBigNumber(this).toString(), this.currency);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get a simple object representing the current monetary value
     * @returns - object with a string integer value and currency code
     */
    Money.prototype.toJSON = function () {
        return {
            amount: this.amount,
            currency: this.currency.toJSON(),
        };
    };
    /**
     * Check that the currency of the passed value matches the current currency.
     * If not, throw an error.
     * @param value - The money object which is used for currency check
     */
    Money.prototype.checkValueCurrency = function (value) {
        if (!this.hasSameCurrency(value)) {
            throw new CurrencyMismatchError();
        }
    };
    /**
     * Convert the constructor input value to an internal BigNumber instance
     * @param value - integer, integer string, float string, instance of `Money`
     * @param BN - BigNumber constructor used by this "Money" instance
     * @returns - Internal BigNumber instance
     */
    Money.prototype.preProcessInputValue = function (value, BN) {
        if (value instanceof Money) {
            this.checkValueCurrency(value);
            return value.amountAsBigNumber;
        }
        var divisor = getSmallestUnitDivisor(this);
        if (divisor.isGreaterThan(1)
            && ((typeof value === 'string' && isInt(value))
                || Number.isInteger(value))) {
            var bignumber = new BN(value);
            return bignumber
                .dividedBy(divisor)
                .decimalPlaces(this.currency.decimalDigits);
        }
        if (typeof value === 'string' && isFloat(value)) {
            return new BN(value);
        }
        throw new WrongInputError(oneLine(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n      The input value must be either an integer,\n      an integer-like string, a float-like string or a \"Money\" instance.\n      "], ["\n      The input value must be either an integer,\n      an integer-like string, a float-like string or a \"Money\" instance.\n      "]))));
    };
    /**
     * Generate a compatible BigNumber constructor
     * @returns - a new BigNumber constructor
     */
    Money.prototype.generateBigNumberConstructor = function () {
        return BigNumber.clone({
            DECIMAL_PLACES: 20,
            ROUNDING_MODE: ROUNDING.HALF_UP,
            FORMAT: {
                decimalSeparator: this.currency.decimalSeparator,
                groupSeparator: this.currency.thousandsSeparator,
                groupSize: 3,
                secondaryGroupSize: 0,
                fractionGroupSeparator: ' ',
                fractionGroupSize: 0,
            },
        });
    };
    /**
     * Creates a new instance of Money
     * @param value - integer, integer string, float string, instance of `Money`
     * @param currency - currency code as string, currency settings or instance of `Currency`
     */
    Money.init = function (value, currency) {
        var MoneyConstructor = this;
        return new MoneyConstructor(value, currency);
    };
    return Money;
}());
export { Money };
export default Money;
var templateObject_1;
//# sourceMappingURL=index.js.map