"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_isequal_1 = __importDefault(require("lodash.isequal"));
var deep_freeze_1 = __importDefault(require("deep-freeze"));
var CurrencyStore_1 = require("../CurrencyStore");
var WrongInputError_1 = __importDefault(require("../errors/WrongInputError"));
var InvalidCurrencyError_1 = __importDefault(require("../errors/InvalidCurrencyError"));
var getDefaultSettings_1 = __importDefault(require("./getDefaultSettings"));
/**
 * @example
 * Using code
 * ```js
 * let currency = new Currency('USD');
 * ```
 *
 * @example
 * Using existing currency object
 * ```js
 * let currency = new Currency(new Currency('USD'));
 * ```
 *
 * @example
 * Using custom settings to create
 * a one-off currency without registering it
 * ```js
 * let currency = new Currency({
 *  code: 'ETH',
 *  symbol: 'Ξ'
 * });
 * ```
 */
var Currency = /** @class */ (function () {
    /**
     * @param currency - Currency string code, custom settings or instance of Currency
     */
    function Currency(currency) {
        var _a = this.preProcess(currency), thousandsSeparator = _a.thousandsSeparator, decimalSeparator = _a.decimalSeparator, decimalDigits = _a.decimalDigits, pattern = _a.pattern, symbol = _a.symbol, formatter = _a.formatter, parser = _a.parser, code = _a.code;
        this.thousandsSeparator = thousandsSeparator;
        this.decimalSeparator = decimalSeparator;
        this.decimalDigits = decimalDigits;
        this.pattern = pattern;
        this.symbol = symbol;
        this.formatter = formatter;
        this.parser = parser;
        this.code = code;
        deep_freeze_1.default(this);
    }
    /**
     * Check if the parameter currency is the same as the current currency
     * @param currency - Currency string code or instance of Currency
     * @returns - returns true if the parameter currency is the same as the current currency
     */
    Currency.prototype.is = function (currency) {
        var CurrencyConstructor = this.constructor;
        currency = new CurrencyConstructor(currency);
        return lodash_isequal_1.default(this.settings, currency.settings);
    };
    /**
     * Get currency code
     * @returns - Currency code
     */
    Currency.prototype.toString = function () {
        return this.code;
    };
    /**
     * Get currency code
     * @returns - Currency code
     */
    Currency.prototype.toJSON = function () {
        return this.code;
    };
    /**
     * Clone the currency instance
     * @returns - new Currency instance
     */
    Currency.prototype.clone = function () {
        var CurrencyConstructor = this.constructor;
        return new CurrencyConstructor(this.settings);
    };
    Object.defineProperty(Currency.prototype, "settings", {
        /**
         * Get settings
         * @returns - Currency settings
         */
        get: function () {
            return {
                thousandsSeparator: this.thousandsSeparator,
                decimalSeparator: this.decimalSeparator,
                decimalDigits: this.decimalDigits,
                pattern: this.pattern,
                symbol: this.symbol,
                formatter: this.formatter,
                parser: this.parser,
                code: this.code,
            };
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Used by the constructor to pre-process the input
     * @param currency - Currency code, settings or instance of Currency
     * @returns - Currency settings
     */
    Currency.prototype.preProcess = function (currency) {
        var settings = {};
        if (currency instanceof Currency) {
            settings = currency.settings;
        }
        else if (currency && typeof currency === 'string') {
            settings = CurrencyStore_1.getRegisteredCurrency(currency);
        }
        else if (currency && typeof currency === 'object' && !Array.isArray(currency)) {
            if (typeof currency.code !== 'string' || !currency.code) {
                throw new InvalidCurrencyError_1.default('Invalid currency settings; code is required.');
            }
            var defaultSettings = getDefaultSettings_1.default(currency.code);
            if (CurrencyStore_1.isCurrencyRegistered(currency.code)) {
                defaultSettings = __assign({}, defaultSettings, CurrencyStore_1.getRegisteredCurrency(currency.code));
            }
            settings = __assign({}, defaultSettings, currency);
        }
        else {
            throw new WrongInputError_1.default('Invalid currency provided.');
        }
        return settings;
    };
    /**
     * Creates a new instance of Currency
     * @param currency - Currency string code, custom settings or instance of Currency
     */
    Currency.init = function (currency) {
        var CurrencyConstructor = this;
        return new CurrencyConstructor(currency);
    };
    return Currency;
}());
exports.default = Currency;
//# sourceMappingURL=index.js.map